package aufgabe4;

import java.util.ArrayList;
import java.util.List;

/**
 * Generic test class for implementing and testing receipt class
 *                _ _   _               _
 * __      ___ __(_) |_| |_ ___ _ __   | |__  _   _
 * \ \ /\ / / '__| | __| __/ _ \ '_ \  | '_ \| | | |
 *  \ V  V /| |  | | |_| ||  __/ | | | | |_) | |_| |
 *   \_/\_/ |_|  |_|\__|\__\___|_| |_| |_.__/ \__, |
 *                                            |___/
 *  ____                  __     __               _
 * / ___|_   _____ _ __   \ \   / /__   __ _  ___| |
 * \___ \ \ / / _ \ '_ \   \ \ / / _ \ / _` |/ _ \ |
 *  ___) \ V /  __/ | | |   \ V / (_) | (_| |  __/ |
 * |____/ \_/ \___|_| |_|    \_/ \___/ \__, |\___|_|
 *                                     |___/
 * Licensed under the GPLv2 License, Version 2.0 (the "License");
 * Copyright (c) Sven Vogel
 */
public class Receipt {

    private final List<ReceiptItem> items = new ArrayList<>();

    /**
     * Add a new receipt item to the list
     * @param item the item to add
     */
    public void add(ReceiptItem item) {
        this.items.add(item);
    }

    /**
     * Print all receipt items in a linear manner.
     */
    public void print() {
        System.out.println("""
                ╔════════════════════════╗
                ║    SuperShop 042       ║
                ║     Marchstr. 23       ║
                ║    10227 Mannheim      ║
                ║   ☏ 030 314 213 86     ║
                ╚════════════════════════╝""");

        items.forEach(System.out::println);

        var total = items.stream().map(a -> a.getPrice() * a.getAmount()).reduce(Double::sum).orElse(0.0);
        System.out.format("\n\t\t\t=====\nSumme EUR\t%.2f\n\t\t\t=====", total);
    }

    public static void main(String[] args) {
        var r = new Receipt();
        r.add( new ReceiptItem("15 Fischstäbchen", 3, 1.79) );
        r.add( new ReceiptItem("Steaks", 5, 3.99) );
        r.add( new ReceiptItem("Naturelle (1l)", 6, 4.99) );
        r.add( new ReceiptItem("Magnum Eis Creme", 2, 2.99) );

        // output receipt in a linear fashion
        r.print();
    }
}
